/// Adapted from https://github.com/kevmoo/build_version/blob/master/lib/builder.dart

import 'dart:io' show File, Process;

import 'package:build/build.dart';

import 'package:yaml/yaml.dart' show loadYaml;

Builder buildAppInfo([BuilderOptions? options]) =>
    // output is hardcoded in `build.yaml`
    _AppInfoBuilder(options!.config['output']);

class _AppInfoBuilder implements Builder {
  final String output;

  _AppInfoBuilder(this.output);

  @override
  Future build(BuildStep buildStep) async {
    final packageInfo = loadYaml(await File('pubspec.yaml').readAsString());

    String packageGitHash = '';

    try {
      final git = await Process.run('git', ['rev-parse', '--short', 'HEAD']);

      if (git.exitCode == 0) {
        packageGitHash = git.stdout.toString().trim();
      }
    } catch (_) {}

    await buildStep.writeAsString(buildStep.allowedOutputs.single, '''
// DO NOT MODIFY! GENERATED BY `dart run build_runner build`

// Information about the package from pubspec
final packageName = '${packageInfo['name']}';
final packageVersion = '${packageInfo['version']}';
final packageDescription =
  '${packageInfo['description']}';

// git hash
final packageGitHash = '$packageGitHash';
''');
  }

  @override
  Map<String, List<String>> get buildExtensions => {
        'pubspec.yaml': [output],
      };
}
